@@ -15,15 +15,20 @@ class CLI(cl1.CLI):
 		# defaults.
 		cl1.CLI.__init__(self,
 			modes={
+				"Keys:":"*chapter*",
+				"    --generate":"Generate a ssh key.",
+				"        --path /keys/mykey/":"Specify the keys directory path.",
+				"        --passphrase Passphrase123":"Specify the keys passphrase.",
+				"        --comment 'My Key'":"Specify the keys comment.",
 				"Aliases:":"*chapter*",
 				"    --list-aliases":"List all aliases.",
 				"        --joiner ','":"Optionally specify the joiner.",
-				"    --alias myserver":"Select one or multiple aliases (example x,y,z) (or use * [\\*] for all).",
+				"    --alias example.com":"Select one or multiple aliases (example: x,y,z) (or use all to select all aliases).",
 				"        --info":"Show the aliases info.",
-				"        --delete":"Delete an alias.",
+				"        --delete  ":"Delete an alias.",
 				"            -f / --forced":"Ignore the are you sure prompt.",
 				"        --create":"Create an alias.",
-				"            --server myserver":"Specify the server's name.",
+				"            --server example.com":"Specify the server's name.",
 				"            --username myuser":"Specify the username.",
 				"            --ip 0.0.0.0":"Specify the server's ip.",
 				"            --port 22":"Specify the server's port.",
@@ -36,11 +41,6 @@ class CLI(cl1.CLI):
 				"        --edit":"Edit the alias config.",
 				"            *** same options as --create ***":"",
 				"            --alias newalias":"Rename the alias.",
-				"Keys:":"*chapter*",
-				"    --generate":"Generate a ssh key.",
-				"        --path /keys/mykey/":"Specify the keys directory path.",
-				"        --passphrase Passphrase123":"Specify the keys passphrase.",
-				"        --comment 'My Key'":"Specify the keys comment.",
 				"Sessions:":"*chapter*",
 				"    --command <alias> 'ls .'":"Execute a command over ssh.",
 				"    --session <alias>":"Start a ssh session.",
@@ -51,25 +51,46 @@ class CLI(cl1.CLI):
 				"        --safe":"Enable version control.",
 				"        --forced":"Enable forced mode.",
 				"    --push <alias>:<remote> <path>":"Push a file / directory.",
-				"        --delete":"Also update the deleted files (optional).",
-				"        --safe":"Enable version control.",
-				"        --forced":"Enable forced mode.",
+				"        --delete ":"Also update the deleted files (optional).",
+				"        --safe ":"Enable version control.",
+				"        --forced ":"Enable forced mode.",
+				"        --exclude .git,.gitignore ":"Exclude additional subpaths (optioal).",
+				"        --no-exclude":"Skip the default excludes and exlude nothing.",
 				"Mounts:":"*chapter*",
-				"    --mount <alias>:<remote> <path>":"Mount a remote directory.",
-				"    --sync <alias>:<remote> <path>":"Synchronize a remote & local directory (similair to --mount except it never unmounts on neither sides).",
-				"    --unmount <path>":"Unmount a mounted remote directory.",
-				"        --sudo":"Root permission required.",
-				"        --forced":"Enable forced mode.",
-				"    --index <path> / <alias>:<remote>":"Index the specified path / alias:remote.",
+				"    --mount <alias>:<id> <path>":"Mount a remote share.",
+				"        --smb":"Select smb mode (default).",
+				"        --sshfs":"Select sshfs mode (when enabled parameter id becomes remote).",
+				"        *** smb options: ***":"SMB --mount options.",
+				"        --reconnect":"Attempt to reconnect the mount when the connection is lost.",
+				"        --tunnel":"Mount the smb share through a ssh tunnel (overwrites options --port & --ip).",
+				"        --username administrator":"Overwrite the smb user (default is retrieved from alias).",
+				"        --password 'SomePassphrase123'":"Set the password of the smb user login (default is no password '').",
+				"        --port 445":"Select a specific smb port (default is 445).",
+				"        --ip 127.0.0.1":"Select a specific ip (default is retrieved from alias).",
+				"    --unmount <path>":"Unmount a mounted share.",
+				"        --sudo  ":"Root permission required.",
+				"        --forced  ":"Enable forced mode.",
+				"Tunnels:":"*chapter*",
+				"    --list-tunnels [optional: <alias>]":"List all tunnels, optionally pass an alias filter.",
+				"        --joiner ',' ":"Optionally specify the joiner.",
+				"    --tunnel <port>:<ip>:<remote_port>:<alias>":"Select a ssh tunnel.",
+				"        --establish":"Establish the selected ssh tunnel.",
+				"            --reconnect":"Attempt to reconnect the tunnel when the connection is lost.",
+				"            --sleeptime 60":"Set the sleeptime value (default is 60) (only when --reconnect is enabled).",
+				"            --reattempts 15":"Set the reattempts value (default is 15) (only when --reconnect is enabled).",
+				"        --kill":"Kill the selected ssh tunnel.",
+				#"SSync:":"*chapter*",
+				#"    --index <path> / <alias>:<remote>":"Index the specified path / alias:remote.",
+				#"    --sync <alias>:<remote> <path>":"Synchronize a remote & local directory (similair to --mount except it never unmounts on neither sides).",
+				#"Daemons:":"*chapter*",
+				#"    --start-daemon <alias>:<remote> <path>":"Start a ssync daemon manually.",
+				#"    --stop-daemon <path>":"Stop a ssync daemon.",
+				#"    --list-daemons":"List all daemons & their status.",
 				"Agent:":"*chapter*",
+				"    --sync":"Manually synchronize the aliases & add the keys to the agent.",
 				"    --start-agent":"Start the ssht00ls agent manually.",
 				"    --stop-agent":"Stop the ssht00ls agent.",
-				"Daemons:":"*chapter*",
-				"    --start-daemon <alias>:<remote> <path>":"Start a ssync daemon manually.",
-				"    --stop-daemon <path>":"Stop a ssync daemon.",
-				"    --list-daemons":"List all daemons & their status.",
 				"Basic:":"*chapter*",
-				"    --sync":"Manually synchronize the aliases & add the keys to the agent.",
 				"    --kill <identifier>":"Kill all ssh processes that include the identifier.",
 				"    --config":"Edit the ssht00ls configuration file (nano).",
 				"    --reset-cache":"Reset the cache directory.",
@@ -99,7 +120,7 @@ class CLI(cl1.CLI):
 		self.arguments.check(json=syst3m.defaults.options.json, exceptions=["--log-level", "--version", "--create-alias", "--non-interative"])
 
 		# sync aliases.
-		if encryption.activated or self.arguments.present("--sync"):
+		if encryption.activated and not self.arguments.present(["-h", "--config", "--help", "--version", "--unmount", "--list-tunnels"]):
 			response = ssht00ls.aliases.sync()
 			if self.arguments.present("--sync"):
 				self.stop(response=response)
@@ -109,12 +130,8 @@ class CLI(cl1.CLI):
 		# BASICS
 		#
 
-		# help.
-		if self.arguments.present(['-h', '--help']):
-			self.docs(success=True, json=syst3m.defaults.options.json)
-
 		# version.
-		elif self.arguments.present(['--version']):
+		if self.arguments.present(['--version']):
 			self.stop(message=f"{ALIAS} version:"+Files.load(f"{SOURCE_PATH}/.version.py").replace("\n",""), json=syst3m.defaults.options.json)
 
 		# config.
@@ -171,19 +188,26 @@ class CLI(cl1.CLI):
 
 		# list aliases.
 		elif self.arguments.present("--list-aliases"):
-			array = Array(array=ssht00ls.aliases.list())
+			response = ssht00ls.aliases.list()
 			if syst3m.defaults.options.json:
-				print(array.array)
+				print(response.dictionary)
 			else:
-				joiner = self.arguments.get("--joiner", required=False, default="\n")
-				print(array.string(joiner=joiner))
+				if self.arguments.present("--joiner"):
+					print(Array(response.array).string(joiner=self.arguments.get("--joiner", required=False, default="\n")))
+				else:
+					print("Aliases:") ; c = 1
+					for i in response.array: print(f" * {c}: {i}") ; c += 1
 
 		# aliases.
 		elif self.arguments.present("--alias"):
 
+			# help.
+			if self.arguments.present(['-h', '--help']):
+				self.docs(chapter="aliases", success=True, json=syst3m.defaults.options.json)
+
 			# get alias.
 			aliases = self.arguments.get("--alias")
-			all = ssht00ls.aliases.list()
+			all = ssht00ls.aliases.list()["aliases"]
 			all_str = str(all).replace("'","")
 			if aliases in ["*", "all"]: aliases = all
 			else: aliases = aliases.split(",")
@@ -366,10 +390,10 @@ class CLI(cl1.CLI):
 			alias,remote = remote.split(":")
 			remote = syst3m.env.fill(remote)
 			path = syst3m.env.fill(path)
-			exclude = None
+			exclude = []
 			if self.arguments.present("--exclude"): 
-				exclude = self.arguments.get("--exclude", chapter="push & pull", mode="--pull").split(",")
-			elif self.arguments.present("--no-exclude"): exclude = []
+				exclude = self.arguments.get("--exclude", chapter="push & pull", mode="--pull", format=list)
+			if self.arguments.present("--no-exclude"): exclude = None
 			response = ssht00ls.ssync.pull(
 				alias=alias, 
 				remote=remote, 
@@ -397,10 +421,10 @@ class CLI(cl1.CLI):
 			alias,remote = remote.split(":")
 			remote = syst3m.env.fill(remote)
 			path = syst3m.env.fill(path)
-			exclude = None
+			exclude = []
 			if self.arguments.present("--exclude"): 
-				exclude = self.arguments.get("--exclude", chapter="push & pull", mode="--push").split(",")
-			elif self.arguments.present("--no-exclude"): exclude = []
+				exclude = self.arguments.get("--exclude", chapter="push & pull", mode="--pull", format=list)
+			if self.arguments.present("--no-exclude"): exclude = None
 			response = ssht00ls.ssync.push(
 				alias=alias, 
 				remote=remote, 
@@ -409,7 +433,7 @@ class CLI(cl1.CLI):
 				forced=self.arguments.present("--forced"), 
 				delete=self.arguments.present("--delete"), 
 				safe=self.arguments.present("--safe"), 
-				directory=True, )
+				directory=None, )
 			self.stop(response=response, json=syst3m.defaults.options.json)
 
 		#
@@ -418,65 +442,69 @@ class CLI(cl1.CLI):
 
 		# mount.
 		elif self.arguments.present('--mount'):
-			self.stop(error="Coming soon.")
-			remote = self.arguments.get("--mount", index=1, chapter="mounts", mode="--mount", notes={})
-			path = self.arguments.get("--mount", index=2, chapter="mounts", mode="--mount", notes={})
-			if ":" not in remote:
-				self.docs(
-					error=f"Invalid <alias>:<remote> <path>.", 
-					chapter="mounts", 
-					mode="--mount", 
-					notes={
-						"<alias>:<path>":"Pack the alias & tuple together as one argument in the following format [<alias>:<path>]."
-					},
-					json=syst3m.defaults.options.json,)
-			alias,remote = remote.split(":")
-			remote = syst3m.env.fill(remote)
-			path = syst3m.env.fill(path)
-			response = ssht00ls.ssync.mount(
-				alias=alias, 
-				remote=remote, 
-				path=path,
-				forced=self.arguments.present("--forced"), )
-			self.stop(response=response, json=syst3m.defaults.options.json)
-
-		# sync.
-		elif self.arguments.present('--sync'):
-			self.stop(error="Coming soon.")
-			remote = self.arguments.get("--sync", index=1, chapter="mounts", mode="--sync", notes={})
-			path = self.arguments.get("--sync", index=2, chapter="mounts", mode="--sync", notes={})
-			if ":" not in remote:
-				self.docs(
-					error=f"Invalid <alias>:<remote> <path>.", 
-					chapter="mounts", 
-					mode="--sync", 
-					notes={
-						"<alias>:<path>":"Pack the alias & tuple together as one argument in the following format [<alias>:<path>]."
-					},
-					json=syst3m.defaults.options.json,)
-			alias,remote = remote.split(":")
-			remote = syst3m.env.fill(remote)
-			path = syst3m.env.fill(path)
-			response = ssht00ls.ssync.mount(
-				alias=alias, 
-				remote=remote, 
-				path=path,
-				forced=self.arguments.present("--forced"), 
-				mode="sync",)
+			if self.arguments.present("--sshfs"):
+				remote = self.arguments.get("--mount", index=1, chapter="mounts", mode="--mount", notes={})
+				path = self.arguments.get("--mount", index=2, chapter="mounts", mode="--mount", notes={})
+				if ":" not in remote:
+					self.docs(
+						error=f"Invalid <alias>:<remote> <path>.", 
+						chapter="mounts", 
+						mode="--mount", 
+						notes={
+							"<alias>:<remote>":"Pack the alias & remote as a tuple together as one argument in the following format [<alias>:<remote>]."
+						},
+						json=syst3m.defaults.options.json,)
+				alias,remote = remote.split(":")
+				remote = syst3m.env.fill(remote)
+				path = syst3m.env.fill(path)
+				response = ssht00ls.sshfs.mount(
+					alias=alias, 
+					remote=remote, 
+					path=path,
+				)
+			else:
+				alias = self.arguments.get("--mount", index=1, chapter="mounts", mode="--mount", notes={})
+				path = self.arguments.get("--mount", index=2, chapter="mounts", mode="--mount", notes={})
+				if ":" not in alias:
+					self.docs(
+						error=f"Invalid [<alias>:<id> <path>] argument packing.", 
+						chapter="mounts", 
+						mode="--mount", 
+						notes={
+							"<alias>:<id>":"Pack the alias & share id as a tuple together as one argument in the following format [<alias>:<id>]."
+						},
+						json=syst3m.defaults.options.json,)
+				alias,id = alias.split(":")
+				path = syst3m.env.fill(path)
+				response = ssht00ls.smb.mount(
+					id=id, 
+					path=path,
+					alias=alias, 
+					password=self.arguments.get("--password", required=False, default=""),
+					username=self.arguments.get("--username", required=False, default=None),
+					ip=self.arguments.get("--ip", required=False, default=None),
+					port=self.arguments.get("--port", required=False, default=None, format=int),
+					tunnel=self.arguments.present("--tunnel"), 
+					reconnect=self.arguments.present("--reconnect"), 
+				)
 			self.stop(response=response, json=syst3m.defaults.options.json)
 
 		# unmount.
 		elif self.arguments.present('--unmount'):
 			path = self.arguments.get("--unmount", index=1, chapter="mounts", mode="--unmount")
-			response = ssht00ls.ssync.unmount(
+			response = ssht00ls.smb.unmount(
 				path=path,
 				forced=self.arguments.present("--forced"), 
 				sudo=self.arguments.present("--sudo"), )
 			self.stop(response=response, json=syst3m.defaults.options.json)
 
+		#
+		# SSYNC
+		#
+
 		# index.
 		elif self.arguments.present('--index'):
-			index = self.arguments.get("--index", chapter="mounts", mode="--index")
+			index = self.arguments.get("--index", chapter="ssync", mode="--index")
 			if ":" in index:
 				alias,remote = index.split(":")
 				remote = syst3m.env.fill(remote)
@@ -486,52 +514,148 @@ class CLI(cl1.CLI):
 				response = ssht00ls.ssync.index(path=index)
 			self.stop(response=response, json=syst3m.defaults.options.json)
 
+		# sync
+			"""
+			elif self.arguments.present('--sync'):
+				self.stop(error="Coming soon.")
+				remote = self.arguments.get("--sync", index=1, chapter="ssync", mode="--sync", notes={})
+				path = self.arguments.get("--sync", index=2, chapter="ssync", mode="--sync", notes={})
+				if ":" not in remote:
+					self.docs(
+						error=f"Invalid <alias>:<remote> <path>.", 
+						chapter="ssync", 
+						mode="--sync", 
+						notes={
+							"<alias>:<remote>":"Pack the alias & remote as a tuple together as one argument in the following format [<alias>:<remote>]."
+						},
+						json=syst3m.defaults.options.json,)
+				alias,remote = remote.split(":")
+				remote = syst3m.env.fill(remote)
+				path = syst3m.env.fill(path)
+				response = ssht00ls.ssync.mount(
+					alias=alias, 
+					remote=remote, 
+					path=path,
+					forced=self.arguments.present("--forced"), 
+					mode="sync",)
+				self.stop(response=response, json=syst3m.defaults.options.json)
+			"""
+
+		#
+		# TUNNELS
+		#
+
+		# list tunnels.
+		elif self.arguments.present("--list-tunnels"):
+			response = ssht00ls.ssh.tunnel.list(alias=self.arguments.get("--list-tunnels", required=False, default=None))
+			if syst3m.defaults.options.json:
+				print(response.dictionary)
+			else:
+				if self.arguments.present("--joiner"):
+					print(Array(response.array).string(joiner=self.arguments.get("--joiner", required=False, default="\n")))
+				else:
+					print("Tunnels:") ; c = 1
+					for i in response.array: print(f" * {c}: {i}") ; c += 1
+
+		# tunnel.
+		elif self.arguments.present('--tunnel'):
+
+			# help.
+			if self.arguments.present(['-h', '--help']):
+				self.docs(chapter="tunnels", success=True, json=syst3m.defaults.options.json)
+
+			# tunnel.
+			id = self.arguments.get("--tunnel", index=1, chapter="tunnels", notes={})
+			failed = False
+			try: port,ip,remote_port,alias = alias.split(":")
+			except: failed = True
+			if failed:
+				self.docs(
+					error=f"Invalid [<port>:<ip>:<remote_port>:<alias>] argument packing.", 
+					chapter="tunnels", 
+					mode=None, 
+					notes={
+						"<port>:<ip>:<remote_port>:<alias>":"Pack the port, ip, remote port & alias as a tuple together as one argument in the following format [<port>:<ip>:<remote_port>:<alias>]."
+					},
+					json=syst3m.defaults.options.json,)
+			tunnel = ssht00ls.ssh.Tunnel(
+				alias=alias,
+				ip=ip,
+				port=port,
+				remote_port=remote_port,
+				reconnect=self.arguments.present("--reconnect"),
+				sleeptime=self.arguments.get("--sleeptime", required=False, default=60, format=int),
+				reattemps=self.arguments.get("--reattemps", required=False, default=15, format=int),
+				log_level=syst3m.defaults.options.log_level,)
+
+			# establish.
+			if self.arguments.present("--establish"):
+				response = tunnel.establish()
+				self.stop(response=response)
+
+			# kill.
+			elif self.arguments.present("--kill"):
+				response = tunnel.kill()
+				self.stop(response=response)
+
+			# invalid.
+			else: self.invalid(chapter="tunnels")
+
 		#
 		# DAEMONS
 		#
+			"""
+			# start daemon.
+			elif self.arguments.present('--start-daemon'):
+				remote = self.arguments.get("--start-daemon", index=1, chapter="daemons", mode="--start-daemon")
+				path = self.arguments.get("--start-daemon", index=2, chapter="daemons", mode="--start-daemon")
+				if ":" not in remote:
+					self.docs(
+						error=f"Invalid <alias>:<remote> <path>.", 
+						chapter="damons", 
+						mode="--start-daemon", 
+						notes={
+							"<alias>:<path>":"Pack the alias & tuple together as one argument in the following format [<alias>:<path>]."
+						},
+						json=syst3m.defaults.options.json,)
+				alias,remote = remote.split(":")
+				remote = syst3m.env.fill(remote)
+				path = syst3m.env.fill(path)
+				response = ssht00ls.ssync.daemon(alias=alias, remote=remote, path=path)
+				self.stop(response=response, json=syst3m.defaults.options.json)
 
-		# start daemon.
-		elif self.arguments.present('--start-daemon'):
-			remote = self.arguments.get("--start-daemon", index=1, chapter="daemons", mode="--start-daemon")
-			path = self.arguments.get("--start-daemon", index=2, chapter="daemons", mode="--start-daemon")
-			if ":" not in remote:
-				self.docs(
-					error=f"Invalid <alias>:<remote> <path>.", 
-					chapter="damons", 
-					mode="--start-daemon", 
-					notes={
-						"<alias>:<path>":"Pack the alias & tuple together as one argument in the following format [<alias>:<path>]."
-					},
-					json=syst3m.defaults.options.json,)
-			alias,remote = remote.split(":")
-			remote = syst3m.env.fill(remote)
-			path = syst3m.env.fill(path)
-			response = ssht00ls.ssync.daemon(alias=alias, remote=remote, path=path)
-			self.stop(response=response, json=syst3m.defaults.options.json)
+			# stop daemon.
+			elif self.arguments.present('--stop-daemon'):
+				c = 0
+				for path in self.arguments.get("--stop-daemon", index=1, chapter="daemon", mode="--stop-daemon", format=list):
+					response = ssht00ls.ssync.daemons.stop(path)
+					if not response["success"]:
+						self.stop(response=response, json=syst3m.defaults.options.json)
+						c += 1
+				if c > 0:
+					self.stop(message=f"Successfully stopped {c} daemon(s).", json=syst3m.defaults.options.json)
+				else:
+					self.stop(error="No daemons found.", json=syst3m.defaults.options.json)
 
-		# stop daemon.
-		elif self.arguments.present('--stop-daemon'):
-			c = 0
-			for path in self.arguments.get("--stop-daemon", index=1, chapter="daemon", mode="--stop-daemon", format=list):
-				response = ssht00ls.ssync.daemons.stop(path)
-				if not response["success"]:
-					self.stop(response=response, json=syst3m.defaults.options.json)
-					c += 1
-			if c > 0:
-				self.stop(message=f"Successfully stopped {c} daemon(s).", json=syst3m.defaults.options.json)
-			else:
-				self.stop(error="No daemons found.", json=syst3m.defaults.options.json)
+			# list daemons.
+			elif self.arguments.present('--list-daemons'):
+				
+				daemons = ssht00ls.ssync.daemons.status()
+				if len(daemons) == 0:
+					self.stop(message=f"There are no active daemons.", json=syst3m.defaults.options.json)
+				print("Daemons:")
+				for path, status in daemons.items():
+					print(f" * {path}: {status}")
+				self.stop(message=f"Successfully listed {len(daemons)} daemon(s).", json=syst3m.defaults.options.json)
+			"""
 
-		# list daemons.
-		elif self.arguments.present('--list-daemons'):
-			
-			daemons = ssht00ls.ssync.daemons.status()
-			if len(daemons) == 0:
-				self.stop(message=f"There are no active daemons.", json=syst3m.defaults.options.json)
-			print("Daemons:")
-			for path, status in daemons.items():
-				print(f" * {path}: {status}")
-			self.stop(message=f"Successfully listed {len(daemons)} daemon(s).", json=syst3m.defaults.options.json)
+		# 
+		# HELP.
+		#
+
+		# help.
+		elif self.arguments.present(['-h', '--help']):
+			self.docs(success=True, json=syst3m.defaults.options.json)
 
 		# invalid.
 		else: self.invalid()
@@ -541,7 +665,12 @@ class CLI(cl1.CLI):
 		passphrase = self.arguments.get("--passphrase", required=required)
 		if passphrase not in [False, None, "", "null", "None", "none"]: return passphrase.replace("\\", "").replace("\ ", "")
 		else: return passphrase
+
 # main.
 if __name__ == "__main__":
 	cli = CLI()
 	cli.start()
+
+
+
+
