@@ -7,7 +7,7 @@ from ssht00ls.classes import utils
 from ssht00ls.classes.smartcards import smartcards
 
 # the ssh key object class.
-class Keys(object):
+class Keys(syst3m.objects.Traceback):
 	def __init__(self,
 		# initialize as specific not global (optional).
 		#	the username.
@@ -18,6 +18,9 @@ class Keys(object):
 		private_key=None,
 	):
 
+		# defaults.
+		syst3m.objects.Traceback.__init__(self, traceback="ssht00ls.keys", raw_traceback="ssht00ls.classes.keys.Keys")
+
 		# specific variables.
 		self.specific = username != None
 		self.username = username
@@ -39,7 +42,7 @@ class Keys(object):
 			"old":old,
 			"new":new,
 			"path":path,
-		})
+		}, traceback=self.__traceback__(function="edit_passphrase"))
 		if not response["success"]: return response
 		
 		# check dir.
@@ -80,7 +83,7 @@ class Keys(object):
 			"old":old,
 			"passphrase":passphrase,
 			"comment":comment,
-		})
+		}, traceback=self.__traceback__(function="edit_comment"))
 		if not response["success"]: return response
 
 		# check dir.
@@ -120,7 +123,7 @@ class Keys(object):
 		if directory[len(directory)-1] != "/": directory += "/"
 		response = r3sponse.check_parameters({
 			"path":directory,
-		})
+		}, traceback=self.__traceback__(function="generate"))
 		if not response["success"]: return response
 
 		# check arguments.
