@@ -11,13 +11,16 @@ from ssht00ls.classes.ssh import ssh
 from ssht00ls.classes import utils
 
 # the ssync object class.
-class SSync(object):
-	def __init__(self, 
+class SSync(syst3m.objects.Traceback):
+	def __init__(self,
 		# initialize as specific not global (optional).
 		#	the username.
 		alias=None,
 	):
 
+		# defaults.
+		syst3m.objects.Traceback.__init__(self, traceback="ssht00ls.ssync", raw_traceback="ssht00ls.classes.ssync.SSync")
+
 		# specific args.
 		self.specific = alias != None
 		self.alias = alias
@@ -25,136 +28,7 @@ class SSync(object):
 		# defaults.
 		self.utils = ssync_utils
 		self.daemons = daemons
-		if CHECKS and INTERACTIVE:
-			if not cl1.argument_present("--reset-cache"):
-				daemons.sync()
-	def mount(self, 
-		# the local path.
-		path=None, 
-		# the remote path.
-		remote=None, 
-		# ssh alias.
-		alias=None,
-		# forced.
-		forced=False,
-		# exclude.
-		exclude=['__pycache__', '.DS_Store'],
-		# accept new host verification keys.
-		accept_new_host_keys=True,
-		# log level.
-		log_level=0,
-		# the daemon mode.
-		mode="mount",
-	):
-
-		# check specific.
-		if self.specific:
-			if alias == None: alias = self.alias
-
-		# checks.
-		response = r3sponse.check_parameters({
-			"path:str":path,
-			"remote:str":remote,
-			"alias:str":alias,
-			"exclude:list":exclude,
-			"forced:bool":forced,
-			"mode:str":mode,
-			"accept_new_host_keys:bool":accept_new_host_keys, })
-		if not response["success"]: return response
-		if mode in ["synchronize", "sync", "synch"]: mode = "sync"
-		if mode not in ["mount", "sync"]:
-			return r3sponse.error(f"Specified an invalid mode: [{mode}], options: [mount, sync].")
-		cache_path = gfp.absolute(path=gfp.clean(path.split(" (d)")[0], remove_last_slash=True))
-		if mode in ["mount"] and not forced and Files.exists(path):
-			return r3sponse.error(f"Path [{path}] already exists.")
-		elif self.daemons.running(cache_path):
-			return r3sponse.error(f"Path [{path}] is already mounted.")
-		path = gfp.clean(path=path)
-		remote = gfp.clean(path=remote)
-
-		# pull.
-		if mode in ["mount"]:
-			response = self.pull(
-				path=path, 
-				alias=alias, 
-				remote=remote, 
-				exclude=exclude,
-				forced=forced,
-				delete=True,
-				safe=False,
-				log_level=log_level,)
-			if not response["success"]: return response
-		else:
-			# make sure both dirs exist in the end.
-			# raise error when both dirs do not exist at start.
-			response = ssh.utils.test_dir(path=remote, alias=alias)
-			remote_exists, local_exists = True, Files.exists(path)
-			if not response["success"]: 
-				if "does not exist" in response["error"]:
-					remote_exists = False
-				else:
-					return response
-			if not remote_exists and not local_exists:
-				return r3sponse.error(f"Both local and remote directories {path} & {alias}:{remote} do not exist.")
-			if not local_exists:
-				os.system(f"mkdir -p {path}")
-				if not Files.exists(path): 
-					return r3sponse.error(f"Failed to create directory {path}.")
-			if not remote_exists:
-				response = ssh.utils.test_dir(path=remote, alias=alias, create=True)
-				if not response["success"]: 
-					return r3sponse.error(f"Failed to create directory {alias}:{remote}, error: {response['error']}")
-
-		# start daemon.
-		cache.set(id=cache_path, group="daemons", data=f"*running* (timestamp={Date().seconds_timestamp})")
-		self.set_mounted_icon(path)
-		if not self.daemons.running(cache_path):
-			return r3sponse.error(f"Failed to set the {path} daemon status.")
-		response =  self.daemon(alias=alias, path=path, remote=remote, start=True, mode=mode)
-		if not response.success: return response
-
-		# handler.
-		if mode == "sync":
-			return r3sponse.success(f"Successfully synchronized {alias}:{remote} to {path}.")
-		else:
-			return r3sponse.success(f"Successfully mounted {alias}:{remote} to {path}.")
-
-		#
-	def unmount(self, 
-		# the local path.
-		path=None, 
-		# forced required.
-		forced=False,
-		# sudo required.
-		sudo=False,
-		# log level.
-		log_level=0,
-	):
-
-		# checks.
-		response = r3sponse.check_parameters({
-			"path:str":path,
-			"forced:bool":forced,
-			"sudo:bool":sudo, })
-		if not response["success"]: return response
-		path = gfp.clean(path)
-		cache_path = gfp.absolute(gfp.clean(path.split(" (d)")[0], remove_last_slash=True))
-		if not self.daemons.running(cache_path):
-			if not Files.exists(path):
-				return r3sponse.error(f"Path [{path}] does not exist.")
-			elif not os.path.isdir(path):
-				return r3sponse.error(f"Path [{path}] is not a directory.")
-			status = str(cache.get(id=cache_path, group="daemons"))
-			if not self.daemons.running(cache_path):
-				return r3sponse.error(f"Path [{path}] is not mounted (status: {status}).")
-
-		# wait for daemon stop.
-		response = self.daemons.stop(path=path)
-		if not success: return response
-
-		# handler.
-		return r3sponse.success(f"Successfully unmounted [{path}].")
-
+		
 		#
 	def index(self, path=None, alias=None, log_level=0, checks=True, accept_new_host_keys=True):
 
@@ -277,8 +151,8 @@ class SSync(object):
 		alias=None, 
 		# the remote path.
 		remote=None, 
-		# exlude subpaths (list) (leave None to use default).
-		exclude=None,
+		# exlude subpaths (list) (leave None to exclude none).
+		exclude=[],
 		# path is directory boolean (leave None to parse automatically).
 		directory=True,
 		empty_directory=False,
@@ -354,8 +228,8 @@ class SSync(object):
 		alias=None, 
 		# the remote path.
 		remote=None, 
-		# exlude subpaths (list) (leave None to use default).
-		exclude=None,
+		# exlude subpaths (list) (leave None to exclude none).
+		exclude=[],
 		# path is directory boolean (leave None to parse automatically).
 		directory=True,
 		empty_directory=False,
@@ -426,7 +300,147 @@ class SSync(object):
 			log_level=log_level,
 			# get the command in str.
 			command=command,)
-	# initialize a daemon.
+	# depricated.
+	def mount(self, 
+		# the local path.
+		path=None, 
+		# the remote path.
+		remote=None, 
+		# ssh alias.
+		alias=None,
+		# forced.
+		forced=False,
+		# exclude.
+		exclude=['__pycache__', '.DS_Store'],
+		# accept new host verification keys.
+		accept_new_host_keys=True,
+		# log level.
+		log_level=0,
+		# the daemon mode.
+		mode="mount",
+	):
+
+		# depricated.
+		return r3sponse.error("DEPRICATED.")
+		
+		# check specific.
+		if self.specific:
+			if alias == None: alias = self.alias
+
+		# checks.
+		response = r3sponse.check_parameters(
+			traceback=self.__traceback__(function="mount"), 
+			parameters={
+				"path:str":path,
+				"remote:str":remote,
+				"alias:str":alias,
+				"exclude:list":exclude,
+				"forced:bool":forced,
+				"mode:str":mode,
+				"accept_new_host_keys:bool":accept_new_host_keys, 
+			})
+		if not response["success"]: return response
+		if mode in ["synchronize", "sync", "synch"]: mode = "sync"
+		if mode not in ["mount", "sync"]:
+			return r3sponse.error(f"Specified an invalid mode: [{mode}], options: [mount, sync].")
+		cache_path = gfp.absolute(path=gfp.clean(path.split(" (d)")[0], remove_last_slash=True))
+		if mode in ["mount"] and not forced and Files.exists(path):
+			return r3sponse.error(f"Path [{path}] already exists.")
+		elif self.daemons.running(cache_path):
+			return r3sponse.error(f"Path [{path}] is already mounted.")
+		path = gfp.clean(path=path)
+		remote = gfp.clean(path=remote)
+
+		# pull.
+		if mode in ["mount"]:
+			response = self.pull(
+				path=path, 
+				alias=alias, 
+				remote=remote, 
+				exclude=exclude,
+				forced=forced,
+				delete=True,
+				safe=False,
+				log_level=log_level,)
+			if not response["success"]: return response
+		else:
+			# make sure both dirs exist in the end.
+			# raise error when both dirs do not exist at start.
+			response = ssh.utils.test_dir(path=remote, alias=alias)
+			remote_exists, local_exists = True, Files.exists(path)
+			if not response["success"]: 
+				if "does not exist" in response["error"]:
+					remote_exists = False
+				else:
+					return response
+			if not remote_exists and not local_exists:
+				return r3sponse.error(f"Both local and remote directories {path} & {alias}:{remote} do not exist.")
+			if not local_exists:
+				os.system(f"mkdir -p {path}")
+				if not Files.exists(path): 
+					return r3sponse.error(f"Failed to create directory {path}.")
+			if not remote_exists:
+				response = ssh.utils.test_dir(path=remote, alias=alias, create=True)
+				if not response["success"]: 
+					return r3sponse.error(f"Failed to create directory {alias}:{remote}, error: {response['error']}")
+
+		# start daemon.
+		cache.set(id=cache_path, group="daemons", data=f"*running* (timestamp={Date().seconds_timestamp})")
+		self.set_mounted_icon(path)
+		if not self.daemons.running(cache_path):
+			return r3sponse.error(f"Failed to set the {path} daemon status.")
+		response =  self.daemon(alias=alias, path=path, remote=remote, start=True, mode=mode)
+		if not response.success: return response
+
+		# handler.
+		if mode == "sync":
+			return r3sponse.success(f"Successfully synchronized {alias}:{remote} to {path}.")
+		else:
+			return r3sponse.success(f"Successfully mounted {alias}:{remote} to {path}.")
+
+		#
+	def unmount(self, 
+		# the local path.
+		path=None, 
+		# forced required.
+		forced=False,
+		# sudo required.
+		sudo=False,
+		# log level.
+		log_level=0,
+	):
+
+		# depricated.
+		return r3sponse.error("DEPRICATED.")
+
+		# checks.
+		response = r3sponse.check_parameters(
+			traceback=self.__traceback__(function="unmount"), 
+			parameters={
+				"path:str":path,
+				"forced:bool":forced,
+				"sudo:bool":sudo, 
+			})
+		if not response["success"]: return response
+		path = gfp.clean(path)
+		cache_path = gfp.absolute(gfp.clean(path.split(" (d)")[0], remove_last_slash=True))
+		if not self.daemons.running(cache_path):
+			if not Files.exists(path):
+				return r3sponse.error(f"Path [{path}] does not exist.")
+			elif not os.path.isdir(path):
+				return r3sponse.error(f"Path [{path}] is not a directory.")
+			status = str(cache.get(id=cache_path, group="daemons"))
+			if not self.daemons.running(cache_path):
+				return r3sponse.error(f"Path [{path}] is not mounted (status: {status}).")
+
+		# wait for daemon stop.
+		response = self.daemons.stop(path=path)
+		if not success: return response
+
+		# handler.
+		return r3sponse.success(f"Successfully unmounted [{path}].")
+
+		#
 	def daemon(self, 
 		# the ssh alias.
 		alias=None, 
@@ -447,6 +461,9 @@ class SSync(object):
 		# serialized.
 		serialized={},
 	):
+		# depricated.
+		return r3sponse.error("DEPRICATED.")
+
 		# check specific.
 		if self.specific:
 			if alias == None: alias = self.alias
@@ -479,10 +496,13 @@ class SSync(object):
 			"utils":self.utils,
 			"sleeptime":sleeptime,
 		})
-		if start: webserver.start_daemon(_daemon_, group="daemons", id=_daemon_.id)
+		if start: webserver.start_thread(_daemon_, group="daemons", id=_daemon_.id)
 		return r3sponse.success("Successfully initialized the daemon", {
 			"daemon":_daemon_,
 		})
 	
 # initialized objects.
 ssync = SSync()
+if CHECKS and INTERACTIVE and not cl1.argument_present("--reset-cache"):
+	daemons.sync()
+	
\ No newline at end of file
