@@ -51,10 +51,10 @@ def pull(
 	alias=None, 
 	# the remote path.
 	remote=None, 
-	# exlude subpaths (list) (leave None to use default).
-	exclude=None,
-	# include subpaths (list) (leave None to use default).
-	include=None,
+	# exlude subpaths (list) (leave None to exclude none).
+	exclude=[],
+	# include subpaths (list) (leave None to include none).
+	include=[],
 	# path is directory boolean (leave None to parse automatically).
 	directory=True,
 	empty_directory=False,
@@ -74,8 +74,8 @@ def pull(
 	command=False,
 ):	
 	# checks.
-	if exclude == None: exclude = EXCLUDE
-	if include == None: include = INCLUDE
+	if exclude != None: exclude += EXCLUDE
+	if include != None: include += INCLUDE
 	if checks:
 
 		# check alias.
@@ -160,10 +160,10 @@ def push(
 	alias=None, 
 	# the remote path.
 	remote=None, 
-	# exlude subpaths (list) (leave None to use default).
-	exclude=None,
-	# include subpaths (list) (leave None to use default).
-	include=None,
+	# exlude subpaths (list) (leave None to exclude none).
+	exclude=[],
+	# include subpaths (list) (leave None to include none).
+	include=[],
 	# path is directory boolean (leave None to parse automatically).
 	directory=True,
 	empty_directory=False,
@@ -184,8 +184,8 @@ def push(
 	command=False,
 ):
 	# checks.
-	if exclude == None: exclude = EXCLUDE
-	if include == None: include = INCLUDE
+	if exclude != None: exclude += EXCLUDE
+	if include != None: include += INCLUDE
 	if checks:
 
 		# check alias.
